/*
 * @Description: pdf预览、内容高亮插件
 * @Author: lmh
 * @Date: 2024-05-17 14:16:50
 * @LastEditTime: 2024-06-05 10:41:06
 * @LastEditors: lmh
 */
import { GlobalWorkerOptions, getDocument } from 'pdfjs-dist/legacy/build/pdf.mjs'
// @ts-ignore
import PdfWorker from 'pdfjs-dist/legacy/build/pdf.worker.min.mjs?url'
// const pdfWorkerMin = require('./pdf.worker.min.js')
// const pdfjsLib = require('./pdf.min.js')
// const pdfjsLib = await import('pdfjs-dist/build/pdf');
// const pdfWorkerMin = await import('pdfjs-dist/build/pdf.worker.entry');
// @ts-ignore
// import PdfjsWorker from "pdfjs-dist/build/pdf.worker.js";
// import pdfjsLib from 'pdfjs-dist'
// @ts-ignore
// import pdfWorkerMin from 'pdfjs-dist/build/pdf.worker.entry'
// import pdfWorkerMin from 'pdfjs-dist/build/pdf.worker.min.js'
// const pdfWorkerMin = import('pdfjs-dist/build/pdf.worker.entry.js')
// const pdfWorkerMin = 'https://cdnjs.cloudflare.com/ajax/libs/pdf.js/3.7.107/pdf.worker.min.js';
// import 'pdfjs-dist/build/pdf.worker.entry'
// @ts-ignore
// pdfjsLib.GlobalWorkerOptions.workerSrc = (typeof window !== "undefined"
//   ? window
//   : {}
// // @ts-ignore
// ).pdfjsWorker = require("pdfjs-dist/build/pdf.worker.min.js");
// const pdfjsLib = require('pdfjs-dist');
// pdfjsLib.GlobalWorkerOptions.workerSrc = require('pdfjs-dist/build/pdf.worker.entry');
// console.log('pdfjsLib', pdfjsLib)
// @ts-ignore
// pdfjsLib.workerSrc = "pdfjs-dist/build/pdf.worker.js";
// pdfjsLib.GlobalWorkerOptions.workerSrc = 'https://cdnjs.cloudflare.com/ajax/libs/pdf.js/3.7.107/pdf.worker.min.js';
// pdfjsLib.GlobalWorkerOptions.workerPort = new PdfjsWorker();
// pdfjsLib.GlobalWorkerOptions.workerSrc = pdfWorkerMin
// import type { PDFDocumentProxy, PDFPageProxy } from 'pdfjs-dist/types/src/pdf'
import type {
  PDFDocumentProxy,
  PDFPageProxy,
  IPDFViewer,
  IOptions,
  IPdfState,
  CustomCanvasRenderingContext2D,
  ILabelState,
  IChunkBox,
  IChunkBoxes,
  IStyle,
  IItemPanel
} from './types'
import { IDomEnum } from './types.d'

if (!GlobalWorkerOptions?.workerSrc) {
  GlobalWorkerOptions.workerSrc = PdfWorker
}

// pdfjsLib.GlobalWorkerOptions.workerSrc = pdfWorkerMin.default

export default class PDFViewer implements IPDFViewer {
  private options: IOptions
  public container: HTMLElement | null = null
  public scrollList: HTMLElement | null = null
  private url: string = ''
  private pdfDocument: PDFDocumentProxy | null = null
  private pdfState: IPdfState = {
    pageNum: 1, // 当前页面
    scale: 1, // 缩放比例
    numPages: 0, // 总页数
    width: 0, // pdf宽度
    height: 0 // pdf高度
  }
  private style: IStyle = {
    fill: "rgba(255, 236, 61, 0.2)",
    stroke: "#ffec3d",
    strokeWidth: "1",
    width: '',
    height: ''
  }
  // private domMethod: IDomEnum | undefined = IDomEnum.VIRTUAL
  private domMethod: IDomEnum | undefined = IDomEnum.SCROLL
  // 源数据
  private dataSource: IItemPanel[] = []
  // 渲染数据 => 通过定义首位index截取源数据
  private renderData: IItemPanel[] = []
  // +2 撑开listScroll容器使其具有滚动条
  private maxCount = 2
  // 开始位置索引
  private startIndex = 0
  // 结束位置索引
  private endIndex = 0
  // 记录到的位置索引
  private pointerIndex = 0
  private isPageChange: boolean = false;

  // private pdfSrc = 'https://cdnjs.cloudflare.com/ajax/libs/pdf.js/3.7.107/pdf.min.js'
  // private pdfWorkerSrc = 'https://cdnjs.cloudflare.com/ajax/libs/pdf.js/3.7.107/pdf.min.js'

  constructor(options: IOptions) {
    this.options = options
    this.container = options.container

    this.url = options.url
    this.style = {
      ...this.style,
      ...options.style
    }
    if (options.domMethod) this.domMethod = options.domMethod

    // const script1 = document.querySelector(`script[src="${this.pdfSrc}"]`);
    // if(!script1) {
    //   this.insertScript(this.pdfSrc).then(() => {
    //     console.warn('insertScript 成功')
    //     this.insertScript(this.pdfWorkerSrc).then(() => {
          
    //     })
    //   })
    // }

    this.init()

    if (this.container) {
      this.scrollList = document.createElement('div');
      this.scrollList.className = 'scroll-list';
      this.container.appendChild(this.scrollList);
    }

    this.loadFile(options)
    this.container?.addEventListener('scroll', this.handleContainerScroll.bind(this));
  }
  private insertScript(src: string) {
     return new Promise((resolve, reject) => {
        const script = document.createElement('script');
        script.src = src;
        script.onload = resolve;
        script.onerror = reject;
        document.head.appendChild(script);
    });
  }

  private init() {
    this.pdfDocument = null;
    this.pdfState = {
      pageNum: 1,
      scale: 1,
      numPages: 0,
      width: 0,
      height: 0
    }
    this.container!.innerHTML = ''
    this.pageState = this.pdfState;
    this.dataSource = []
    this.renderData = []
    this.maxCount = 2
    this.startIndex = 0
    this.endIndex = 0
    this.pointerIndex = 0
    this.isPageChange = false;
  }
  public destroy() {
    this.init()
    this.container?.removeEventListener('scroll', this.handleContainerScroll.bind(this));
    // const script1 = document.querySelector(`script[src="${this.pdfSrc}"]`);
    // const script2 = document.querySelector(`script[src="${this.pdfWorkerSrc}"]`);
    // if (script1) script1.remove();
    // if (script2) script2.remove();
  }

  private async loadFile(options: IOptions) {
    // pdfjsLib.GlobalWorkerOptions.workerSrc = pdfWorkerMin.default
    const loadingTask = await getDocument({
      url: this.url,
      disableRange: true,
      cMapUrl: 'https://unpkg.com/pdfjs-dist@4.3.136/cmaps/'
    })
    await loadingTask.promise.then((pdf: PDFDocumentProxy) => {
      this.pdfDocument = pdf
      this.pdfState.numPages = pdf.numPages
      options.initCallback && options.initCallback(pdf)
      setTimeout(() => {
        this.renderPage(1)
      }, 300)
    })
  }

  private async renderPage(pageNum: number) {
    await this.pdfDocument?.getPage(pageNum).then(async (page: PDFPageProxy) => {
      const itemPanel = document.createElement('div')
      itemPanel.id = `pdf-page-${pageNum}`
      itemPanel.className = 'pdf-page'
      itemPanel.style.position = 'relative'

      // this.scrollList?.appendChild(itemPanel)
      const canvas = document.createElement('canvas')
      canvas.id = 'vuePdfId' + pageNum
      const ctx = canvas.getContext('2d', { willReadFrequently: true }) as CustomCanvasRenderingContext2D
      const dpr = window.devicePixelRatio || 1
      const bsr =
        ctx?.webkitBackingStorePixelRatio ||
        ctx?.mozBackingStorePixelRatio ||
        ctx?.msBackingStorePixelRatio ||
        ctx?.oBackingStorePixelRatio ||
        ctx?.backingStorePixelRatio ||
        1
      // const ratio = dpr / bsr
      // console.log('dpr / bsr', dpr / bsr)
      const ratio = 1
      const wrapWidth = this.container?.clientWidth || 0
      const viewport = page.getViewport({ scale: dpr / bsr + 1 })

      const scale = wrapWidth / viewport.width
      console.log('scale ==== >', scale)
      this.pdfState.scale = scale
      this.pdfState.width = viewport.width * scale
      this.pdfState.height = viewport.height * scale
      canvas.width = viewport.width * ratio
      canvas.height = viewport.height * ratio
      canvas.style.width = canvas.width * scale + 'px'
      canvas.style.height = canvas.height * scale + 'px'
      this.style.width = canvas.width * scale + 'px'
      this.style.height = canvas.height * scale + 'px'
      itemPanel.style.minWidth = canvas.width * scale + 'px'
      itemPanel.style.height = canvas.height * scale + 10 + 'px'
      const totalHeight = (this.pdfState.height + 10) * this.pdfState.numPages
      this.scrollList!.style.height = totalHeight + 'px'
      // this.container!.style.height = totalHeight + 'px'

      this.maxCount = Math.floor(this.container!.clientHeight / this.pdfState.height) + 2
      // console.log('this.maxCount', this.maxCount)

      ctx?.setTransform(ratio, 0, 0, ratio, 0, 0)
      const renderContext = {
        canvasContext: ctx,
        viewport: viewport
      }
      await page.render(renderContext)

      itemPanel.appendChild(canvas)
      if (this.domMethod === IDomEnum.ALL) {
        this.scrollList?.appendChild(itemPanel)
      } else {
        this.dataSource.push({page: pageNum, item: itemPanel})
        this.Render()
      }
      if (pageNum < this.pdfState.numPages) {
        await this.renderPage(pageNum + 1)
      }
    })
  }
  // 计算开始位置和结束位置索引
  private  ComputePointerPosition () {
    const end = this.startIndex + this.maxCount
    this.endIndex = this.dataSource[end] ? end : this.dataSource.length
  }
  // TODO: 列表需要改成Map()
  // 截取渲染数据
  private  GetRenderData () {
    // console.log('slice', this.startIndex, this.endIndex)
    this.renderData = this.dataSource.sort((a, b) => a.page - b.page).slice(this.startIndex, this.endIndex)
  }
  // 渲染
  private Render () {
    // console.log('this.dataSource', this.dataSource)
    // console.log('this.renderData', this.renderData)
    this.ComputePointerPosition()
    this.GetRenderData()
    if (this.domMethod === IDomEnum.VIRTUAL) this.scrollList!.innerHTML = '';
    this.renderData.forEach(itemPanel => {
      this.scrollList?.appendChild(itemPanel.item)
    })
  }

  private scrollIntoView(labelState: ILabelState[]) {
    if (!labelState.length) return
    // this.container!.scrollTop = this.pdfState.height * labelState[0].page
    const parent = this.container as HTMLElement
    const pageY = labelState[0].page * this.pdfState.height
    parent.scrollTo({
      top: pageY,
      behavior: 'smooth'
    })
  }
  private handleContainerScroll(event: Event) {
    const container = event.target as HTMLElement
    const scrollTop = container?.scrollTop
    const numPage = Math.floor(scrollTop / this.pdfState.height)  + 1;
    if (!this.isPageChange && numPage <= this.pdfState.numPages) {
      this.pdfState.pageNum = numPage
      this.pageState = this.pdfState;
    }
    if (this.domMethod !== IDomEnum.ALL) {
      // 更新开始位置索引：滚动的距离 / 每个元素的高度
      this.startIndex = Math.floor(scrollTop / this.pdfState.height)
      // console.log('1 this.pointerIndex:', this.pointerIndex, 'this.startIndex:', this.startIndex)
      if (this.domMethod === IDomEnum.SCROLL && this.pointerIndex >= this.startIndex) {
        return
      }
      this.pointerIndex = this.startIndex
      // console.log('2 this.pointerIndex:', this.pointerIndex, 'this.startIndex:', this.startIndex)
      // 位置更新，就更新渲染
      this.Render()
      if (this.domMethod === IDomEnum.VIRTUAL && this.dataSource.length - this.startIndex >= this.maxCount) {
        // 测试发现每次向下滚动一个元素，列表会向上移动一个元素的位置，所以增加transform属性，使列表位置向下移动一个元素的位置
        // startIndex表示已经上移到的元素的个数，itemHeight表示每个元素的高度
        this.scrollList!.style.transform = `translateY(${this.startIndex * this.pdfState.height}px)`
        this.scrollList!.style.height = (this.pdfState.height + 10) * this.maxCount + 'px'
      }
    }
  }
  private scrollToCurr() {
    // this.container?.removeEventListener('scroll', this.handleContainerScroll.bind(this))
    this.container!.scrollTop = this.pdfState.height * (this.pdfState.pageNum - 1)
    // setTimeout(() => {
    //   this.container?.addEventListener('scroll', this.handleContainerScroll.bind(this))
    // }, 50)
  }
  private clearPdfPageSvg() {
    const svgs = this.scrollList?.querySelectorAll('svg')
    svgs?.forEach(svg => svg.remove())
  }
  private createPdfPageSvg(svgs:IChunkBoxes[]) {
    svgs.forEach((box: IChunkBoxes) => {
      const pageElement = document.getElementById(`pdf-page-${box.page + 1}`);
      if (!pageElement) return;

      const svg = document.createElementNS("http://www.w3.org/2000/svg", "svg");
      svg.setAttribute("width", this.style.width);
      svg.setAttribute("height", this.style.height);
      svg.style.position = 'absolute';
      svg.style.top = '0';
      svg.style.left = '50%';
      svg.style.transform = 'translateX(-50%)';

      box.bboxes.forEach((bbox: IChunkBox) => {
        const rect = document.createElementNS("http://www.w3.org/2000/svg", "rect");
        rect.setAttribute("x", (bbox[0] * this.pdfState.scale).toString());
        rect.setAttribute("y", (bbox[1] * this.pdfState.scale).toString());
        rect.setAttribute("width", ((bbox[2] - bbox[0]) * this.pdfState.scale).toString());
        rect.setAttribute("height", ((bbox[3] - bbox[1]) * this.pdfState.scale).toString());
        rect.style.fill = this.style.fill;
        rect.style.stroke = this.style.stroke;
        rect.style.strokeWidth = this.style.strokeWidth;
        svg.appendChild(rect);
      });

      pageElement.appendChild(svg);
    });
  }
  private groupByPage(data: ILabelState[]) {
    const groupedData = new Map()
    data.forEach(item => {
      if (!groupedData.has(item.page)) {
        groupedData.set(item.page, {
          page: item.page,
          bboxes: [item.bbox]
        })
      } else {
        groupedData.get(item.page).bboxes.push(item.bbox)
      }
    })
    return [...groupedData.values()]
  }
  public setHighlight(chunk_bboxes: ILabelState[]) {
    const svgs = this.groupByPage(chunk_bboxes)
    if (!svgs.length) return
    const totalHeight = (this.pdfState.height + 10) * this.pdfState.numPages
    this.scrollList!.style.height = totalHeight + 'px'
    this.clearPdfPageSvg()
    // this.jumpPage(svgs[0].page + 1)
    this.scrollIntoView(svgs)
    const time = this.domMethod === IDomEnum.VIRTUAL ? 1000 : this.domMethod === IDomEnum.SCROLL ? 200 : 100
    setTimeout(() => {
      this.createPdfPageSvg(svgs)
    }, time)
  }
  public get pageState() {
    return this.pdfState
  }
  public set pageState(newState: IPdfState) {
    this.pdfState = {
      ...this.pdfState,
      ...newState
    }
    this.options.pdfStateCallback?.(this.pdfState);
  }
  public prePage() {
    if (this.pdfState.pageNum > 1) {
      this.isPageChange = true
      this.pdfState.pageNum -= 1
      this.scrollToCurr()
      this.pageState = this.pdfState;
      setTimeout(() => this.isPageChange = false, 50)
    }
  }
  public jumpPage(num: number) {
    if (0 < num && num <= this.pdfState.numPages) {
      this.isPageChange = true
      this.pdfState.pageNum = num
      this.scrollToCurr()
      this.pageState = this.pdfState;
      setTimeout(() => this.isPageChange = false, 50)
    }
  }
  public nextPage() {
    if (this.pdfState.pageNum < this.pdfState.numPages) {
      this.isPageChange = true
      this.pdfState.pageNum += 1
      this.scrollToCurr()
      this.pageState = this.pdfState;
      setTimeout(() => this.isPageChange = false, 50)
    }
  }
}
